// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.0
// -  \   \        Filename: ERROR_DETECTOR.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// -Revision History:
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

The ERROR_DETECTOR module takes a raw data input from the receiver interface
under test, and synchronizes that data to a "golden" copy of the data.  Once
the two data streams are synchronized, the circuit continuously compares the
receiver data to the "golden" data, keeping track of the bit error count in
the receiver data.  Frame count and synchronization status are also provided       
as outputs.
--------------------------------------------------------------------------------
*/

`timescale 1ps/1ps
module ERROR_DETECTOR
	(
	RXDATA,
	PAT_SEL,
	RESET,
	CLK,
	TRAINING_DONE,
	PATTERN_ADVANCE,
	CAPTURE_FC,
	CAPTURE_BEC,
	FRAME_COUNT,
	ERRORED_BITS,
	ARMED_FLAG,
	LOCKED_FLAG,
	DISTRESS_FLAG,
	LOCAL_GEN_DATA,                 // DE BUGGING 
        STATE,                          // DE BUGGING	
	ABORT_FLAG
	);
	
input	[3:0]	RXDATA;			// DATA UNDER TEST FROM RX INTERFACE
input	[3:0]	PAT_SEL;		// IDENTIFICATION OF INCOMING PATTERN (E.G. PRBS23)
input		RESET;			// RESETS ALL DETECTION LOGIC AND ERROR COUNTERS
input		CLK;			// CLK
input           TRAINING_DONE;          // TRAINING DONE
input		PATTERN_ADVANCE;	// ALLOWS USER TO HALT THE "GOLDEN" DATA PATTERN 
input		CAPTURE_FC;		// STROBE TO CAPTURE FRAME COUNT
input		CAPTURE_BEC;		// STROBE TO CAPTURE BIT ERROR COUNT

output	[31:0]	FRAME_COUNT;		// NUMBER OF FRAMES RECEIVED SINCE SYNC ACHIEVED
output	[31:0]	ERRORED_BITS;		// RUNNING COUNT OF BIT ERRORS IN RXDATA
output	reg	ARMED_FLAG;		// ARMED STATE INDICATES SYNC HAS NOT BEEN ACHIEVED
output	reg	LOCKED_FLAG;		// LOCKED STATE INDICATES RXDATA IS SYNCHRONIZED
output	reg	DISTRESS_FLAG;		// ONE OR MORE ERRORS FOUND IN A FRAME
output  [3:0]   LOCAL_GEN_DATA;         // DE BUGGING
output  [13:0]  STATE;                  // DE BUGGING
output  reg	ABORT_FLAG;		// ONE OR MORE ERRORS FOUND IN 4 CONSECUTIVE FRAMES 

wire	[3:0]	PATTERN_TO_COMPARE;	// GOLDEN DATA
wire	[3:0]	ERROR_CMP;		// RESULT OF XOR BETWEEN RXDATA AND "GOLDEN" DATA		

reg	[4:0]	CS; //synthesis attribute safe_implementation of CS is "yes";
reg	[4:0]	NS;
reg	[3:0]	RXDATA_TO_DETECTOR_PIPE;
reg	[3:0]	RXDATA_TO_DETECTOR_PIPE2;
reg	[31:0]	FRAME_COUNT;
reg	[31:0]	FRAME_COUNT_PRELIM;
reg	[31:0]	ERRORED_BITS;
reg	[31:0]	ERRORED_BITS_PRELIM;
reg	[3:0]	ERROR_CMP_REG;
reg	[3:0]	ERROR_SUM;

parameter	IDLE		= 0;
parameter	ARM		= 1;
parameter	ARM2		= 2;
parameter	ARM3		= 3;
parameter	ARM4		= 4;
parameter	ARM5		= 5;
parameter	ARM6		= 6;
parameter	ARM7		= 7;
parameter	ARM8		= 8;
parameter	ARM9   		= 9;
parameter	ARM10		= 10; 
parameter	ARM11	        = 11;
parameter	LOCKED		= 12;
parameter	DISTRESS	= 13;
parameter	DISTRESS2	= 14;
parameter	DISTRESS3	= 15;
parameter	ABORT		= 16;

assign PAT_ADVANCE		= (CS == IDLE) | (PATTERN_ADVANCE & ~(CS == ARM));
assign LOCAL_GEN_DATA           = PATTERN_TO_COMPARE;
assign STATE                    = {9'b0,CS};

//GOLDEN PATTERN GENERATOR IN RECEIVER TO COMPARE AGAINST INCOMING DATA
PATTERN_GENERATOR PAT_COMP                                          
	(                                                              
	.DATA_OUT(PATTERN_TO_COMPARE),                            
	.PAT_SEL(PAT_SEL),                                             
	.PAT_ADVANCE(PAT_ADVANCE),     
	.ERROR_INSERT(1'b0),	
	.RST(RESET),                                                  
	.CLK(CLK)                                                   
	);
	
//CURRENT STATE LOGIC
always @(posedge CLK) begin
	if (RESET)
		CS <= IDLE;
	else
		CS <= NS;
	end

//NEXT STATE LOGIC
always @(CS or PAT_ADVANCE or PATTERN_TO_COMPARE or RXDATA_TO_DETECTOR_PIPE2 or RXDATA_TO_DETECTOR_PIPE or RXDATA or TRAINING_DONE) begin
	case (CS)
	IDLE:		begin
			if (TRAINING_DONE)
		                NS <= ARM;
			else
				NS <= IDLE;

		        end
				
	ARM:		begin
			if (PATTERN_TO_COMPARE != RXDATA)
				NS <= ARM;
			else
				NS <= ARM2;
			end
			
	ARM2:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE)
				NS <= ARM;
			else
				NS <= ARM3;
			end
	        	
	ARM3:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM3;
			else
				NS <= ARM4;
			end
	        	
	ARM4:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM4;
			else
				NS <= ARM5;
			end
	        	
	ARM5:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM5;
			else
				NS <= ARM6;
			end
	        	
	ARM6:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM6;
			else
				NS <= ARM7;
			end
			
	ARM7:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM7;
			else
				NS <= ARM8;
			end

	ARM8:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM8;
			else
				NS <= ARM9;
			end

	ARM9:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM9;
			else
				NS <= ARM10;
			end

	ARM10:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM10;
			else
				NS <= ARM11;
			end	
        ARM11:		begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ARM;
			else
			if (PAT_ADVANCE == 1'b0)
				NS <= ARM11;
			else
				NS <= LOCKED;
			end				
	LOCKED: 	begin
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= DISTRESS;
			else
				NS <= LOCKED;
			end
			
	DISTRESS:	begin
			if (PAT_ADVANCE == 1'b0)
				NS <= DISTRESS;
			else
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= DISTRESS2;
			else
				NS <= LOCKED;
			end	
	
	DISTRESS2:	begin
			if (PAT_ADVANCE == 1'b0)
				NS <= DISTRESS;
			else
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= DISTRESS3;
			else
				NS <= LOCKED;
			end	
	
	DISTRESS3:	begin
			if (PAT_ADVANCE == 1'b0)
				NS <= DISTRESS;
			else
			if (PATTERN_TO_COMPARE != RXDATA_TO_DETECTOR_PIPE2)
				NS <= ABORT;
			else
				NS <= LOCKED;
			end
		
	ABORT:		begin
			NS <= ABORT;
			end
		
	default: 	NS <= IDLE;
	endcase
end				


//OUTPUT LOGIC
always @(posedge CLK) begin
	case (CS)
	IDLE:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
				
	ARM:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
			
	ARM2:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
	        	
	ARM3:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
	        	
	ARM4:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
	        	
	ARM5:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
	        	
	ARM6:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
			
	ARM7:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end

	ARM8:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end

	ARM9:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end			

	ARM10:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
			
	ARM11:		begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b1;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
	LOCKED: 	begin
			FRAME_COUNT_PRELIM <=	FRAME_COUNT_PRELIM + 1;
			ERRORED_BITS_PRELIM <=	ERRORED_BITS_PRELIM + ERROR_SUM;
			ERROR_SUM	<=	ERROR_CMP_REG[3] + ERROR_CMP_REG[2] + ERROR_CMP_REG[1] + ERROR_CMP_REG[0];
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b1;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
			
	DISTRESS:	begin
			FRAME_COUNT_PRELIM <=	FRAME_COUNT_PRELIM + 1;
			ERRORED_BITS_PRELIM <=	ERRORED_BITS_PRELIM + ERROR_SUM;
			ERROR_SUM	<=	ERROR_CMP_REG[3] + ERROR_CMP_REG[2] + ERROR_CMP_REG[1] + ERROR_CMP_REG[0];
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b1;
			ABORT_FLAG <= 1'b0;
			end	
	
	DISTRESS2:	begin
			FRAME_COUNT_PRELIM <=	FRAME_COUNT_PRELIM + 1;
			ERRORED_BITS_PRELIM <=	ERRORED_BITS_PRELIM + ERROR_SUM;
			ERROR_SUM	<=	ERROR_CMP_REG[3] + ERROR_CMP_REG[2] + ERROR_CMP_REG[1] + ERROR_CMP_REG[0];
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b1;
			ABORT_FLAG <= 1'b0;
			end	
	
	DISTRESS3:	begin
			FRAME_COUNT_PRELIM <=	FRAME_COUNT_PRELIM + 1;
			ERRORED_BITS_PRELIM <=	ERRORED_BITS_PRELIM + ERROR_SUM;
			ERROR_SUM	<=	ERROR_CMP_REG[3] + ERROR_CMP_REG[2] + ERROR_CMP_REG[1] + ERROR_CMP_REG[0];
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b1;
			ABORT_FLAG <= 1'b0;
			end
		
	ABORT:		begin
			FRAME_COUNT_PRELIM <=	FRAME_COUNT_PRELIM + 1;
			ERRORED_BITS_PRELIM <=	ERRORED_BITS_PRELIM + ERROR_SUM;
			ERROR_SUM	<=	ERROR_CMP_REG[3] + ERROR_CMP_REG[2] + ERROR_CMP_REG[1] + ERROR_CMP_REG[0];
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b1;
			end
		
	default:	begin
			FRAME_COUNT_PRELIM <= 32'h00000000;
			ERRORED_BITS_PRELIM <= 32'h00000000;
			ERROR_SUM <= 4'h0;
			ARMED_FLAG <= 1'b0;
			LOCKED_FLAG <= 1'b0;
			DISTRESS_FLAG <= 1'b0;
			ABORT_FLAG <= 1'b0;
			end
	endcase
end				

//PIPELINED VERSIONS OF RECEIVED DATA
always @(posedge CLK) begin
	RXDATA_TO_DETECTOR_PIPE <= RXDATA;
	RXDATA_TO_DETECTOR_PIPE2 <= RXDATA_TO_DETECTOR_PIPE;
	end             

//INCOMING DATA IS CONSTANTLY XOR-ED WITH EXPECTED DATA TO GENERATE AN ERROR COUNT
generate
genvar k;
   for (k=0; k<=3; k=k+1)
   begin :  ERROR_CMP_GEN
      assign ERROR_CMP[k] = RXDATA_TO_DETECTOR_PIPE2[k] ^ PATTERN_TO_COMPARE[k];
   end
endgenerate             

always @(posedge CLK) begin
	ERROR_CMP_REG <= ERROR_CMP;
	end                          

//FRAME COUNT IS CAPTURED WHENEVER THE "CAPTURE_FC" PIN IS ASSERTED.  
//THIS ALLOWS SLOWER CLOCK DOMAINS TO READ THE DATA WITHOUT DANGER OF METASTABILITY 
always @ (posedge CLK) begin
	if (CAPTURE_FC)
		FRAME_COUNT <= FRAME_COUNT_PRELIM;
	else
		FRAME_COUNT <= FRAME_COUNT;
end

//BIT ERROR COUNT IS CAPTURED WHENEVER THE "CAPTURE_BEC" PIN IS ASSERTED.  
//THIS ALLOWS SLOWER CLOCK DOMAINS TO READ THE DATA WITHOUT DANGER OF METASTABILITY 
always @ (posedge CLK) begin
	if (CAPTURE_BEC)
		ERRORED_BITS <= ERRORED_BITS_PRELIM;
	else
		ERRORED_BITS <= ERRORED_BITS;
end

endmodule
