// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.0
// -  \   \        Filename: PATTERN_GENERATOR.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// -Revision History:
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

The PATTERN_GENERATOR module outputs one of several patterns, which are
selected using the PAT_SEL input. There are 4 types of PRBS available, each of
which is an LFSR defined according to ITU specifications.  For each cycle of 
the clock, 8 bits of the LFSR are calculated and placed on DATA_OUT[7:0]
        
 Pattern Menu (by PAT_SEL setting):
 
 4'b0000: Clock Pattern (10101010)
 4'b0001: Clock Pattern (11001100)
 4'b0010: PRBS3 (00101100) This pattern is also the training pattern for RX
 4'b0011: PRBS7
 4'b0100: PRBS15
 4'b0101: PRBS23
 4'b0110: PRBS29
 4'b0111: User Defined (11111110)
--------------------------------------------------------------------------------
*/	

module PATTERN_GENERATOR
	(
	DATA_OUT,
	PAT_SEL,
	PAT_ADVANCE,
	ERROR_INSERT,
	RST,
	CLK
	);

input	[3:0]	PAT_SEL;		//SELECTS THE PATTERN TO BE PRODUCED
input		PAT_ADVANCE;		//PAUSE/RESUME PATTERN GENERATION
input		ERROR_INSERT;           //ERROR INSERT PULSE
input		RST;			
input		CLK;			//8 BITS OF PATTERN PRODUCED EACH CYCLE

output	[3:0]	DATA_OUT;		//DATA_OUT OF SELECTED PATTERN

wire	[3:0]	PRBS7_TO_OSERDES;	//PRBS7 CIRCUIT PRODUCES PATTERN	
wire	[3:0]	PRBS15_TO_OSERDES;      //PRBS15 CIRCUIT PRODUCES PATTERN
wire	[3:0]	PRBS23_TO_OSERDES;	//PRBS23 CIRCUIT PRODUCES PATTERN
wire	[3:0]	PRBS29_TO_OSERDES;	//PRBS29 CIRCUIT PRODUCES PATTERN
reg	[3:0] DATA_OUT_TMP;

parameter	CLOCK_SEL 	= 4'b0000,
		CLOCK2_SEL	= 4'b0001,
		PRBS3_SEL 	= 4'b0010,
		PRBS7_SEL 	= 4'b0011,
		PRBS15_SEL	= 4'b0100,
		PRBS23_SEL	= 4'b0101,          
		PRBS29_SEL	= 4'b0110,
		USER_SEL	= 4'b0111;

always @(posedge CLK) begin
	case (PAT_SEL)
		CLOCK_SEL:	DATA_OUT_TMP <= 4'b1010;
		CLOCK2_SEL:	DATA_OUT_TMP <= 4'b1100;
		PRBS3_SEL:	DATA_OUT_TMP <= 4'b1100;
		PRBS7_SEL:	DATA_OUT_TMP <= PRBS7_TO_OSERDES;
		PRBS15_SEL:	DATA_OUT_TMP <= PRBS15_TO_OSERDES;
		PRBS23_SEL:	DATA_OUT_TMP <= PRBS23_TO_OSERDES;
		PRBS29_SEL:	DATA_OUT_TMP <= PRBS29_TO_OSERDES;
		USER_SEL:	DATA_OUT_TMP <= 4'b1110;
		default:	DATA_OUT_TMP <= 4'b1010;
	endcase
end

assign DATA_OUT = ERROR_INSERT ? ~DATA_OUT_TMP : DATA_OUT_TMP;

PRBSGen07 PRBSGen07_TX
	(
	.data_out(PRBS7_TO_OSERDES),
	.advance_in(PAT_ADVANCE),
	.reset_in(RST),
	.poly_in(7'h03),	//LFSR: F = x^7 + x^6
	.length_in(1'b0),
	.clock_in(CLK)
	);	

PRBSGen15 PRBSGen15_TX
	(
	.data_out(PRBS15_TO_OSERDES),
	.advance_in(PAT_ADVANCE),
	.reset_in(RST),
	.poly_in(15'h003),	//LFSR: F = x^15 + x^14
	.length_in(1'b0),
	.clock_in(CLK)
	);

PRBSGen23 PRBSGen23_TX
	(
	.data_out(PRBS23_TO_OSERDES),
	.advance_in(PAT_ADVANCE),
	.reset_in(RST),
	.poly_in(23'h000021),	//LFSR: F = x^23 + x^18
	.length_in(1'b0),
	.clock_in(CLK)
	);

PRBSGen29 PRBSGen29_TX
	(
	.data_out(PRBS29_TO_OSERDES),
	.advance_in(PAT_ADVANCE),
	.reset_in(RST),
	.poly_in(29'h00000005),	//LFSR: F = x^29 + x^27
	.length_in(1'b0),
	.clock_in(CLK)
	);

endmodule
