// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.0
// -  \   \        Filename: SYSTEM_LOGIC.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// -Revision History:
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

The SYSTEM_LOGIC module operates on a separate domain from the TX and RX
interfaces. FIFO's are included to transfer data to the TX and RX
domains.  Pattern Generators and Error Detectors produce and analyze data 
within the system domain, producing statistics that report up the hierarchy.
--------------------------------------------------------------------------------
*/

module SYSTEM_LOGIC
	(
	//LVDS INTERFACE DATA AND CLOCKS
	DATA_TO_OSERDES,
	DATA_FROM_ISERDES,
	RX_WRCLK,
	TX_RDCLK,
	TX_RST,
	RX_RST,
	
	//SYSTEM CLOCK and CONTROLS
	SYSCLK,
	SYSRST,
	DETECTOR_RST,
	ERROR_INSERT,
	PAT_SEL,
	CAPTURE_FC,
	ERROR_COUNT_CHAN_SEL,
        TRAINING_DONE,
	IDELAY_READY,
	
	//LVDS INTERFACE FLAGS AND STATS
	ARMED_FLAG,
	LOCKED_FLAG,
	DISTRESS_FLAG,
	ABORT_FLAG,
	ERRORED_BITS,
	DATA_SAMPLED,  //debug
	FRAME_COUNT
	);

input	[63:0]	DATA_FROM_ISERDES;
input		RX_WRCLK;
input		TX_RDCLK;
input           TX_RST;
input		RX_RST;
input		SYSCLK;
input		SYSRST;
input		DETECTOR_RST;
input		ERROR_INSERT;
input	[3:0]	PAT_SEL;	
input		CAPTURE_FC;
input 	[3:0]	ERROR_COUNT_CHAN_SEL;
input           TRAINING_DONE;
input           IDELAY_READY;

output	[63:0]	DATA_TO_OSERDES;
output	[15:0]	ARMED_FLAG;
output	[15:0]	LOCKED_FLAG;
output	[15:0]	DISTRESS_FLAG;
output	[15:0]	ABORT_FLAG;
output	[31:0]	ERRORED_BITS;
output	[31:0]	FRAME_COUNT;
output  [95:0]  DATA_SAMPLED;

wire	[63:0]	TXDATA_TO_FIFO;
wire    [13:0]  CURRENT_STATE;
//FIFO 0
wire		AEMPTY_RX_0;
wire		AFULL_RX_0;
wire		EMPTY_RX_0;
wire		FULL_RX_0;
wire		RDEN_RX_0;
wire		WREN_RX_0;
wire		AEMPTY_TX_0;
wire		AFULL_TX_0;
wire		EMPTY_TX_0;
wire		FULL_TX_0;
wire		RDEN_TX_0;
wire		WREN_TX_0;
//FIFO 1
wire		AEMPTY_RX_1;
wire		AFULL_RX_1;
wire		EMPTY_RX_1;
wire		FULL_RX_1;
wire		RDEN_RX_1;
wire		WREN_RX_1;
wire		AEMPTY_TX_1;
wire		AFULL_TX_1;
wire		EMPTY_TX_1;
wire		FULL_TX_1;
wire		RDEN_TX_1;
wire		WREN_TX_1;
//FIFO 2
wire		AEMPTY_RX_2;
wire		AFULL_RX_2;
wire		EMPTY_RX_2;
wire		FULL_RX_2;
wire		RDEN_RX_2;
wire		WREN_RX_2;
wire		AEMPTY_TX_2;
wire		AFULL_TX_2;
wire		EMPTY_TX_2;
wire		FULL_TX_2;
wire		RDEN_TX_2;
wire		WREN_TX_2;

reg		WREN_TX_0_PIPE;
reg		WREN_TX_1_PIPE;
reg		WREN_TX_2_PIPE;
reg	[3:0]	DET_RESET_CAPTURE = 0;
reg		DET_RESET_PULSE = 0;

integer		I;

localparam RST_SYNC_NUM = 3;

// Internal signals
reg [RST_SYNC_NUM-1:0]     rx_rst_sync_reg;
reg [RST_SYNC_NUM-1:0]     tx_rst_sync_reg;
reg 			   stretch_tx_rst_reg;
reg			   stretch_rx_rst_reg;
wire                       stretch_tx_rst;
wire                       stretch_rx_rst;
wire			   tx_fifo_rst;
wire			   rx_fifo_rst;

//****TRANSMITTER LOGIC****************************************************//

wire		pattern_advance_TX_0;
wire		pattern_advance_TX_1;
wire		pattern_advance_TX_2;

assign		pattern_advance_TX_0 = WREN_TX_0; 
assign		pattern_advance_TX_1 = WREN_TX_1; 
assign		pattern_advance_TX_2 = WREN_TX_2; 

PATTERN_GENERATOR PATGEN_TX_00
	(
	.DATA_OUT(TXDATA_TO_FIFO[3:0]),
	.PAT_SEL(PAT_SEL),
	.PAT_ADVANCE(pattern_advance_TX_0),
	.ERROR_INSERT(ERROR_INSERT),
	.RST(SYSRST),
	.CLK(SYSCLK)
	);

PATTERN_GENERATOR PATGEN_TX_01
	(
	.DATA_OUT(TXDATA_TO_FIFO[27:24]),
	.PAT_SEL(PAT_SEL),
	.PAT_ADVANCE(pattern_advance_TX_1),
	.ERROR_INSERT(ERROR_INSERT),
	.RST(SYSRST),
	.CLK(SYSCLK)
	);

PATTERN_GENERATOR PATGEN_TX_02
	(
	.DATA_OUT(TXDATA_TO_FIFO[51:48]),
	.PAT_SEL(PAT_SEL),
	.PAT_ADVANCE(pattern_advance_TX_2),
	.ERROR_INSERT(ERROR_INSERT),
	.RST(SYSRST),
	.CLK(SYSCLK)
	);


assign TXDATA_TO_FIFO[7:4]   = TXDATA_TO_FIFO[3:0];	//CH1
assign TXDATA_TO_FIFO[11:8]  = TXDATA_TO_FIFO[3:0];	//CH2
assign TXDATA_TO_FIFO[15:12] = TXDATA_TO_FIFO[3:0];	//CH3
assign TXDATA_TO_FIFO[19:16] = TXDATA_TO_FIFO[3:0];     //CH4
assign TXDATA_TO_FIFO[23:20] = TXDATA_TO_FIFO[3:0];	//CH5

assign TXDATA_TO_FIFO[31:28] = TXDATA_TO_FIFO[27:24];	//CH7
assign TXDATA_TO_FIFO[35:32] = TXDATA_TO_FIFO[27:24];	//CH8
assign TXDATA_TO_FIFO[39:36] = TXDATA_TO_FIFO[27:24];	//CH9
assign TXDATA_TO_FIFO[43:40] = TXDATA_TO_FIFO[27:24];	//CH10
assign TXDATA_TO_FIFO[47:44] = TXDATA_TO_FIFO[27:24];	//CH11

assign TXDATA_TO_FIFO[55:52] = TXDATA_TO_FIFO[51:48];	//CH13
assign TXDATA_TO_FIFO[59:56] = TXDATA_TO_FIFO[51:48];	//CH14
assign TXDATA_TO_FIFO[63:60] = TXDATA_TO_FIFO[51:48];	//CH15

//****RECEIVER LOGIC*******************************************************//	

wire		pattern_advance_RX_0;
wire		pattern_advance_RX_1;
wire		pattern_advance_RX_2;

reg		pattern_advance_RX_pipe1_0;
reg		pattern_advance_RX_pipe1_1;
reg		pattern_advance_RX_pipe1_2;

reg	[31:0]	ERRORED_BITS;

(* KEEP = "TRUE" *)
wire	[63:0]	RXDATA_TO_DETECTOR;
wire	[31:0]	ERRORED_BITS_ED00;
wire	[31:0]	ERRORED_BITS_ED01;
wire	[31:0]	ERRORED_BITS_ED02;
wire	[31:0]	ERRORED_BITS_ED03;
wire	[31:0]	ERRORED_BITS_ED04;
wire	[31:0]	ERRORED_BITS_ED05;
wire	[31:0]	ERRORED_BITS_ED06;
wire	[31:0]	ERRORED_BITS_ED07;
wire	[31:0]	ERRORED_BITS_ED08;
wire	[31:0]	ERRORED_BITS_ED09;
wire	[31:0]	ERRORED_BITS_ED10;
wire	[31:0]	ERRORED_BITS_ED11;
wire	[31:0]	ERRORED_BITS_ED12;
wire	[31:0]	ERRORED_BITS_ED13;
wire	[31:0]	ERRORED_BITS_ED14;
wire	[31:0]	ERRORED_BITS_ED15;

wire    [3:0]   LOCAL_GEN_DATA_4;
wire    [3:0]   LOCAL_GEN_DATA_6;
wire    [3:0]   LOCAL_GEN_DATA_12;
wire	[7:0]	TEMP0;
wire	[7:0]	TEMP1;

assign pattern_advance_RX_0 = RDEN_RX_0;
assign pattern_advance_RX_1 = RDEN_RX_1;
assign pattern_advance_RX_2 = RDEN_RX_2;

always @(posedge SYSCLK)
    begin
    pattern_advance_RX_pipe1_0 <= pattern_advance_RX_0;
    pattern_advance_RX_pipe1_1 <= pattern_advance_RX_1;
    pattern_advance_RX_pipe1_2 <= pattern_advance_RX_2;
    end

//READ BIT ERRORS ON EACH CHANNEL IN ROUND ROBIN FASHION
always @ (posedge SYSCLK) begin
	case (ERROR_COUNT_CHAN_SEL)
	4'b0000: ERRORED_BITS <= ERRORED_BITS_ED00;
	4'b0001: ERRORED_BITS <= ERRORED_BITS_ED01;
	4'b0010: ERRORED_BITS <= ERRORED_BITS_ED02;
	4'b0011: ERRORED_BITS <= ERRORED_BITS_ED03;
	4'b0100: ERRORED_BITS <= ERRORED_BITS_ED04;
	4'b0101: ERRORED_BITS <= ERRORED_BITS_ED05;
	4'b0110: ERRORED_BITS <= ERRORED_BITS_ED06;
	4'b0111: ERRORED_BITS <= ERRORED_BITS_ED07;
	4'b1000: ERRORED_BITS <= ERRORED_BITS_ED08;
	4'b1001: ERRORED_BITS <= ERRORED_BITS_ED09;
	4'b1010: ERRORED_BITS <= ERRORED_BITS_ED10;
	4'b1011: ERRORED_BITS <= ERRORED_BITS_ED11;
	4'b1100: ERRORED_BITS <= ERRORED_BITS_ED12;
	4'b1101: ERRORED_BITS <= ERRORED_BITS_ED13;
	4'b1110: ERRORED_BITS <= ERRORED_BITS_ED14;
	4'b1111: ERRORED_BITS <= ERRORED_BITS_ED15;
	default: ERRORED_BITS <= ERRORED_BITS;
	endcase
end

ERROR_DETECTOR ERROR_DETECTOR_00
	(
	.RXDATA(RXDATA_TO_DETECTOR[3:0]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_0),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(FRAME_COUNT),
	.ERRORED_BITS(ERRORED_BITS_ED00),
	.ARMED_FLAG(ARMED_FLAG[0]),
	.LOCKED_FLAG(LOCKED_FLAG[0]),
	.DISTRESS_FLAG(DISTRESS_FLAG[0]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[0])
	);

ERROR_DETECTOR ERROR_DETECTOR_01
	(
	.RXDATA(RXDATA_TO_DETECTOR[7:4]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_0),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED01),
	.ARMED_FLAG(ARMED_FLAG[1]),
	.LOCKED_FLAG(LOCKED_FLAG[1]),
	.DISTRESS_FLAG(DISTRESS_FLAG[1]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[1])
	);

ERROR_DETECTOR ERROR_DETECTOR_02
	(
	.RXDATA(RXDATA_TO_DETECTOR[11:8]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_0),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED02),
	.ARMED_FLAG(ARMED_FLAG[2]),
	.LOCKED_FLAG(LOCKED_FLAG[2]),
	.DISTRESS_FLAG(DISTRESS_FLAG[2]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[2])
	);

ERROR_DETECTOR ERROR_DETECTOR_03
	(
	.RXDATA(RXDATA_TO_DETECTOR[15:12]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_0),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED03),
	.ARMED_FLAG(ARMED_FLAG[3]),
	.LOCKED_FLAG(LOCKED_FLAG[3]),
	.DISTRESS_FLAG(DISTRESS_FLAG[3]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[3])
	);

ERROR_DETECTOR ERROR_DETECTOR_04
	(
	.RXDATA(RXDATA_TO_DETECTOR[19:16]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_0),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED04),
	.ARMED_FLAG(ARMED_FLAG[4]),
	.LOCKED_FLAG(LOCKED_FLAG[4]),
	.DISTRESS_FLAG(DISTRESS_FLAG[4]),
	.LOCAL_GEN_DATA(LOCAL_GEN_DATA_4),	// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[4])
	);

ERROR_DETECTOR ERROR_DETECTOR_05
	(
	.RXDATA(RXDATA_TO_DETECTOR[23:20]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_0),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED05),
	.ARMED_FLAG(ARMED_FLAG[5]),
	.LOCKED_FLAG(LOCKED_FLAG[5]),
	.DISTRESS_FLAG(DISTRESS_FLAG[5]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[5])
	);

ERROR_DETECTOR ERROR_DETECTOR_06
	(
	.RXDATA(RXDATA_TO_DETECTOR[27:24]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_1),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED06),
	.ARMED_FLAG(ARMED_FLAG[6]),
	.LOCKED_FLAG(LOCKED_FLAG[6]),
	.DISTRESS_FLAG(DISTRESS_FLAG[6]),
	.LOCAL_GEN_DATA(LOCAL_GEN_DATA_6),	// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[6])
	);

ERROR_DETECTOR ERROR_DETECTOR_07
	(
	.RXDATA(RXDATA_TO_DETECTOR[31:28]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_1),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED07),
	.ARMED_FLAG(ARMED_FLAG[7]),
	.LOCKED_FLAG(LOCKED_FLAG[7]),
	.DISTRESS_FLAG(DISTRESS_FLAG[7]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[7])
	);

ERROR_DETECTOR ERROR_DETECTOR_08
	(
	.RXDATA(RXDATA_TO_DETECTOR[35:32]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_1),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED08),
	.ARMED_FLAG(ARMED_FLAG[8]),
	.LOCKED_FLAG(LOCKED_FLAG[8]),
	.DISTRESS_FLAG(DISTRESS_FLAG[8]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[8])
	);

ERROR_DETECTOR ERROR_DETECTOR_09
	(
	.RXDATA(RXDATA_TO_DETECTOR[39:36]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_1),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED09),
	.ARMED_FLAG(ARMED_FLAG[9]),
	.LOCKED_FLAG(LOCKED_FLAG[9]),
	.DISTRESS_FLAG(DISTRESS_FLAG[9]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[9])
	);

ERROR_DETECTOR ERROR_DETECTOR_10
	(
	.RXDATA(RXDATA_TO_DETECTOR[43:40]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_1),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED10),
	.ARMED_FLAG(ARMED_FLAG[10]),
	.LOCKED_FLAG(LOCKED_FLAG[10]),
	.DISTRESS_FLAG(DISTRESS_FLAG[10]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[10])
	);

ERROR_DETECTOR ERROR_DETECTOR_11
	(
	.RXDATA(RXDATA_TO_DETECTOR[47:44]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_1),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED11),
	.ARMED_FLAG(ARMED_FLAG[11]),
	.LOCKED_FLAG(LOCKED_FLAG[11]),
	.DISTRESS_FLAG(DISTRESS_FLAG[11]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[11])
	);

ERROR_DETECTOR ERROR_DETECTOR_12
	(
	.RXDATA(RXDATA_TO_DETECTOR[51:48]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_2),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED12),
	.ARMED_FLAG(ARMED_FLAG[12]),
	.LOCKED_FLAG(LOCKED_FLAG[12]),
	.DISTRESS_FLAG(DISTRESS_FLAG[12]),
        .LOCAL_GEN_DATA(LOCAL_GEN_DATA_12),	// DEBUG
	.STATE (CURRENT_STATE),			// DEBUG
	.ABORT_FLAG(ABORT_FLAG[12])
	);

ERROR_DETECTOR ERROR_DETECTOR_13
	(
	.RXDATA(RXDATA_TO_DETECTOR[55:52]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_2),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED13),
	.ARMED_FLAG(ARMED_FLAG[13]),
	.LOCKED_FLAG(LOCKED_FLAG[13]),
	.DISTRESS_FLAG(DISTRESS_FLAG[13]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[13])
	);

ERROR_DETECTOR ERROR_DETECTOR_14
	(
	.RXDATA(RXDATA_TO_DETECTOR[59:56]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_2),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED14),
	.ARMED_FLAG(ARMED_FLAG[14]),
	.LOCKED_FLAG(LOCKED_FLAG[14]),
	.DISTRESS_FLAG(DISTRESS_FLAG[14]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[14])
	);

ERROR_DETECTOR ERROR_DETECTOR_15
	(
	.RXDATA(RXDATA_TO_DETECTOR[63:60]),
	.PAT_SEL(PAT_SEL),
	.RESET(DET_RESET_PULSE),
	.CLK(SYSCLK),
	.TRAINING_DONE(TRAINING_DONE),
	.PATTERN_ADVANCE(pattern_advance_RX_pipe1_2),
	.CAPTURE_FC(CAPTURE_FC),
	.CAPTURE_BEC(1'b1),
	.FRAME_COUNT(),
	.ERRORED_BITS(ERRORED_BITS_ED15),
	.ARMED_FLAG(ARMED_FLAG[15]),
	.LOCKED_FLAG(LOCKED_FLAG[15]),
	.DISTRESS_FLAG(DISTRESS_FLAG[15]),
	.LOCAL_GEN_DATA(),			// DEBUG
	.STATE (),				// DEBUG
	.ABORT_FLAG(ABORT_FLAG[15])
	);

//SHORTEN DET_RESET PULSE TO ONE SYSCLK CYCLE
always @(posedge SYSCLK)
   begin
      DET_RESET_CAPTURE[0] <= DETECTOR_RST;                      // asynchronous entry point
      begin
         for(I = 0; I <= 3 - 1; I = I + 1)
         begin
            DET_RESET_CAPTURE[I + 1] <= DET_RESET_CAPTURE[I];    // metastable flip-flops
         end
      end
      DET_RESET_PULSE <= DET_RESET_CAPTURE[2] & ~DET_RESET_CAPTURE[3]; //stable, single pulse
   end

//****FIFO LOGIC**********************************************************//

//FIFO ASSIGNMENTS
assign WREN_TX_0 = stretch_tx_rst ? 1'b0 : ~AFULL_TX_0;
assign RDEN_TX_0 = stretch_tx_rst ? 1'b0 : ~AEMPTY_TX_0;
assign RDEN_RX_0 = stretch_rx_rst ? 1'b0 : ~AEMPTY_RX_0;
assign WREN_RX_0 = stretch_rx_rst ? 1'b0 : ~AFULL_RX_0;

assign WREN_TX_1 = stretch_tx_rst ? 1'b0 : ~AFULL_TX_1;
assign RDEN_TX_1 = stretch_tx_rst ? 1'b0 : ~AEMPTY_TX_1;
assign RDEN_RX_1 = stretch_rx_rst ? 1'b0 : ~AEMPTY_RX_1;
assign WREN_RX_1 = stretch_rx_rst ? 1'b0 : ~AFULL_RX_1;

assign WREN_TX_2 = stretch_tx_rst ? 1'b0 : ~AFULL_TX_2;
assign RDEN_TX_2 = stretch_tx_rst ? 1'b0 : ~AEMPTY_TX_2;
assign RDEN_RX_2 = stretch_rx_rst ? 1'b0 : ~AEMPTY_RX_2;
assign WREN_RX_2 = stretch_rx_rst ? 1'b0 : ~AFULL_RX_2;


always @(posedge SYSCLK)
      begin
      WREN_TX_0_PIPE <= WREN_TX_0;
      WREN_TX_1_PIPE <= WREN_TX_1;
      WREN_TX_2_PIPE <= WREN_TX_2;
      end

fifo_generator_v5_3 FIFO_TX_0 (
	.rst		(tx_fifo_rst),
	.wr_clk		(SYSCLK),
	.rd_clk		(TX_RDCLK),
	.din		(TXDATA_TO_FIFO[23:0]),
	.wr_en		(WREN_TX_0_PIPE),
	.rd_en		(RDEN_TX_0),
	.dout		(DATA_TO_OSERDES[23:0]),
	.full		(FULL_TX_0),
	.almost_full	(AFULL_TX_0),
	.empty		(EMPTY_TX_0),
	.almost_empty	(AEMPTY_TX_0)
  );

fifo_generator_v5_3 FIFO_TX_1 (
	.rst		(tx_fifo_rst),
	.wr_clk		(SYSCLK),
	.rd_clk		(TX_RDCLK),
	.din		(TXDATA_TO_FIFO[47:24]),
	.wr_en		(WREN_TX_1_PIPE),
	.rd_en		(RDEN_TX_1),
	.dout		(DATA_TO_OSERDES[47:24]),
	.full		(FULL_TX_1),
	.almost_full	(AFULL_TX_1),
	.empty		(EMPTY_TX_1),
	.almost_empty	(AEMPTY_TX_1)
  );

fifo_generator_v5_3 FIFO_TX_2 (
	.rst		(tx_fifo_rst),
	.wr_clk		(SYSCLK),
	.rd_clk		(TX_RDCLK),
	.din		({8'b0 , TXDATA_TO_FIFO[63:48]}),
	.wr_en		(WREN_TX_2_PIPE),
	.rd_en		(RDEN_TX_2),
	.dout		({TEMP0 , DATA_TO_OSERDES[63:48]}),
	.full		(FULL_TX_2),
	.almost_full	(AFULL_TX_2),
	.empty		(EMPTY_TX_2),
	.almost_empty	(AEMPTY_TX_2)
  );

fifo_generator_v5_3 FIFO_RX_0 (
	.rst		(rx_fifo_rst),
	.wr_clk		(RX_WRCLK),
	.rd_clk		(SYSCLK),
	.din		(DATA_FROM_ISERDES[23:0]),
	.wr_en		(WREN_RX_0),
	.rd_en		(RDEN_RX_0),
	.dout		(RXDATA_TO_DETECTOR[23:0]),
	.full		(FULL_RX_0),
	.almost_full	(AFULL_RX_0),
	.empty		(EMPTY_RX_0),
	.almost_empty	(AEMPTY_RX_0)
  );

fifo_generator_v5_3 FIFO_RX_1 (
	.rst		(rx_fifo_rst),
	.wr_clk		(RX_WRCLK),
	.rd_clk		(SYSCLK),
	.din		(DATA_FROM_ISERDES[47:24]),
	.wr_en		(WREN_RX_1),
	.rd_en		(RDEN_RX_1),
	.dout		(RXDATA_TO_DETECTOR[47:24]),
	.full		(FULL_RX_1),
	.almost_full	(AFULL_RX_1),
	.empty		(EMPTY_RX_1),
	.almost_empty	(AEMPTY_RX_1)
  );

fifo_generator_v5_3 FIFO_RX_2 (
	.rst		(rx_fifo_rst),
	.wr_clk		(RX_WRCLK),
	.rd_clk		(SYSCLK),
	.din		({8'b0 , DATA_FROM_ISERDES[63:48]}),
	.wr_en		(WREN_RX_2),
	.rd_en		(RDEN_RX_2),
	.dout		({TEMP1 , RXDATA_TO_DETECTOR[63:48]}),
	.full		(FULL_RX_2),
	.almost_full	(AFULL_RX_2),
	.empty		(EMPTY_RX_2),
	.almost_empty	(AEMPTY_RX_2)
  );


always @(posedge RX_WRCLK)
    if (RX_RST)
      rx_rst_sync_reg <= {RST_SYNC_NUM{1'b1}};
    else
      // logical left shift by one (pads with 0)
      rx_rst_sync_reg <= rx_rst_sync_reg << 1;

assign stretch_rx_rst    = rx_rst_sync_reg[RST_SYNC_NUM-1];

always @ (posedge RX_WRCLK)
    stretch_rx_rst_reg <= stretch_rx_rst;

assign rx_fifo_rst = stretch_rx_rst_reg & ~stretch_rx_rst;
  
always @(posedge TX_RDCLK)
    if (TX_RST)
      tx_rst_sync_reg <= {RST_SYNC_NUM{1'b1}};
    else
      // logical left shift by one (pads with 0)
      tx_rst_sync_reg <= tx_rst_sync_reg << 1;

assign stretch_tx_rst    = tx_rst_sync_reg[RST_SYNC_NUM-1];

always @ (posedge TX_RDCLK)
    stretch_tx_rst_reg <= stretch_tx_rst;

assign tx_fifo_rst = stretch_tx_rst_reg & ~stretch_tx_rst;
 

// Debug signals for Chipscope
assign DATA_SAMPLED[3:0]   = RXDATA_TO_DETECTOR[19:16];
assign DATA_SAMPLED[7:4]   = LOCAL_GEN_DATA_4;
assign DATA_SAMPLED[8]     = TRAINING_DONE; 
assign DATA_SAMPLED[9]     = LOCKED_FLAG[4]; 
assign DATA_SAMPLED[13:10] = RXDATA_TO_DETECTOR[27:24];
assign DATA_SAMPLED[17:14] = LOCAL_GEN_DATA_6;
assign DATA_SAMPLED[18]    = 1'b0;//WRERR2; 
assign DATA_SAMPLED[19]    = LOCKED_FLAG[6]; 
assign DATA_SAMPLED[20]    = pattern_advance_RX_pipe1_0;
assign DATA_SAMPLED[21]    = pattern_advance_RX_pipe1_1;
assign DATA_SAMPLED[25:22] = RXDATA_TO_DETECTOR[51:48];
assign DATA_SAMPLED[29:26] = LOCAL_GEN_DATA_12;
assign DATA_SAMPLED[30]    = LOCKED_FLAG[12]; 
assign DATA_SAMPLED[31]    = pattern_advance_RX_pipe1_2;
assign DATA_SAMPLED[32]    = SYSRST;
assign DATA_SAMPLED[33]    = DET_RESET_PULSE;
assign DATA_SAMPLED[34]    = 1'b0;//RDERR0;
assign DATA_SAMPLED[38:35] = TXDATA_TO_FIFO[51:48];
assign DATA_SAMPLED[39]    = AEMPTY_RX_1;
assign DATA_SAMPLED[40]    = AEMPTY_RX_0;
assign DATA_SAMPLED[41]    = AEMPTY_RX_2;
assign DATA_SAMPLED[57:42] = LOCKED_FLAG;
assign DATA_SAMPLED[61:58] = DATA_TO_OSERDES[19:16];
assign DATA_SAMPLED[65:62] = DATA_FROM_ISERDES[19:16];
assign DATA_SAMPLED[71:66] = 6'b0;
assign DATA_SAMPLED[84:72] = 13'b0;
assign DATA_SAMPLED[92:85] = 8'b0;
assign DATA_SAMPLED[93]    = tx_fifo_rst;
assign DATA_SAMPLED[94]    = rx_fifo_rst;
assign DATA_SAMPLED[95]    = 1'b0;//RDERR1;

endmodule



